<?php
namespace app\helpers;

use yii\helpers\ArrayHelper;

class Transliterator
{
    public static function convert(string $string):string
    {
        $converter = array(
            'а' => 'a',    'б' => 'b',    'в' => 'v',    'г' => 'g',    'д' => 'd',
            'е' => 'e',    'ё' => 'e',    'ж' => 'zh',   'з' => 'z',    'и' => 'i',
            'й' => 'y',    'к' => 'k',    'л' => 'l',    'м' => 'm',    'н' => 'n',
            'о' => 'o',    'п' => 'p',    'р' => 'r',    'с' => 's',    'т' => 't',
            'у' => 'u',    'ф' => 'f',    'х' => 'h',    'ц' => 'c',    'ч' => 'ch',
            'ш' => 'sh',   'щ' => 'sch',  'ь' => '',     'ы' => 'y',    'ъ' => '',
            'э' => 'e',    'ю' => 'yu',   'я' => 'ya',

            'А' => 'A',    'Б' => 'B',    'В' => 'V',    'Г' => 'G',    'Д' => 'D',
            'Е' => 'E',    'Ё' => 'E',    'Ж' => 'Zh',   'З' => 'Z',    'И' => 'I',
            'Й' => 'Y',    'К' => 'K',    'Л' => 'L',    'М' => 'M',    'Н' => 'N',
            'О' => 'O',    'П' => 'P',    'Р' => 'R',    'С' => 'S',    'Т' => 'T',
            'У' => 'U',    'Ф' => 'F',    'Х' => 'H',    'Ц' => 'C',    'Ч' => 'Ch',
            'Ш' => 'Sh',   'Щ' => 'Sch',  'Ь' => '',     'Ы' => 'Y',    'Ъ' => '',
            'Э' => 'E',    'Ю' => 'Yu',   'Я' => 'Ya',
        );

        $value = strtr($string, $converter);
        return $value;
    }

    public static function toUrl(string $string):string
    {
        $translit = self::convert($string);
        return strtolower(preg_replace("/([^a-zA-Z0-9-])/","",trim(str_replace(" ",'-',$translit))));
    }

    /**
     * Склонение существительных после числительных.
     *
     * @param string $value Значение
     * @param array $words Массив вариантов, например: array('товар', 'товара', 'товаров')
     * @param bool $show Включает значение $value в результирующею строку
     * @return string
     */
    public static function plural(string $value, array $words, bool $show = true,$zeroPlaceholder=NULL,$onePlaceholder=NULL)
    {
        if($value==0 && !is_null($zeroPlaceholder)) return $zeroPlaceholder;
        if($value==1 && !is_null($onePlaceholder)) return $onePlaceholder;

        $num = $value % 100;
        if ($num > 19) {
            $num = $num % 10;
        }

        $out = ($show) ?  $value . ' ' : '';
        switch ($num) {
            case 1:  $out .= $words[0]; break;
            case 2:
            case 3:
            case 4:  $out .= $words[1]; break;
            default: $out .= $words[2]; break;
        }

        return $out;
    }

    public static function month($number)
    {
        $months = [
            1=>"Января",
            2=>"Февраля",
            3=>"Марта",
            4=>"Апреля",
            5=>"Мая",
            6=>"Июня",
            7=>"Июля",
            8=>"Августа",
            9=>"Сентября",
            10=>"Октября",
            11=>"Ноября",
            12=>"Декабря",
        ];
        return ArrayHelper::getValue($months,$number,$number);
    }
}
